@@ -15,13 +15,13 @@
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
 use crate::*;
+use arc_swap::ArcSwap;
 use snarkos_parameters::GenesisBlock;
 use snarkvm_algorithms::{merkle_tree::MerkleTree, traits::LoadableMerkleParameters};
-use snarkvm_objects::{errors::StorageError, Block, DatabaseTransaction, LedgerScheme, Op, Storage, Transaction};
+use snarkvm_dpc::{errors::StorageError, Block, DatabaseTransaction, LedgerScheme, Op, Storage, TransactionScheme};
 use snarkvm_parameters::{traits::genesis::Genesis, LedgerMerkleTreeParameters, Parameter};
 use snarkvm_utilities::bytes::FromBytes;
 
-use parking_lot::RwLock;
 use std::{
     fs,
     marker::PhantomData,
@@ -34,15 +34,15 @@ use std::{
 
 pub type BlockHeight = u32;
 
-pub struct Ledger<T: Transaction, P: LoadableMerkleParameters, S: Storage> {
+pub struct Ledger<T: TransactionScheme, P: LoadableMerkleParameters, S: Storage> {
     pub current_block_height: AtomicU32,
     pub ledger_parameters: Arc<P>,
-    pub cm_merkle_tree: RwLock<MerkleTree<P>>,
+    pub cm_merkle_tree: ArcSwap<MerkleTree<P>>,
     pub storage: S,
     pub _transaction: PhantomData<T>,
 }
 
-impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
+impl<T: TransactionScheme, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
     /// Create a fresh blockchain, optionally at the specified path.
     /// Warning: if specified, any existing storage at that location is removed.
     pub fn new_empty<PATH: AsRef<Path>>(path: Option<PATH>) -> Result<Self, StorageError> {
@@ -51,10 +51,10 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
 
             Self::open_at_path(path)
         } else {
-            let crh = P::H::from(FromBytes::read(&LedgerMerkleTreeParameters::load_bytes()?[..])?);
+            let crh = P::H::from(FromBytes::read_le(&LedgerMerkleTreeParameters::load_bytes()?[..])?);
             let ledger_parameters = Arc::new(P::from(crh));
 
-            let genesis_block: Block<T> = FromBytes::read(GenesisBlock::load_bytes().as_slice())?;
+            let genesis_block: Block<T> = FromBytes::read_le(GenesisBlock::load_bytes().as_slice())?;
 
             Ok(Self::new(None, ledger_parameters, genesis_block).expect("Ledger could not be instantiated"))
         }
@@ -89,6 +89,16 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
         self.get_current_block_height() + 1
     }
 
+    /// Get the height of the best block on the chain.
+    pub fn get_best_block_number(&self) -> Result<BlockHeight, StorageError> {
+        let best_block_number_bytes = self
+            .storage
+            .get(COL_META, KEY_BEST_BLOCK_NUMBER.as_bytes())?
+            .ok_or_else(|| StorageError::Message("Can't obtain the best block's number".into()))?;
+
+        Ok(bytes_to_u32(&best_block_number_bytes))
+    }
+
     /// Get the stored old connected peers.
     pub fn get_peer_book(&self) -> Result<Option<Vec<u8>>, StorageError> {
         self.storage.get(COL_META, &KEY_PEER_BOOK.as_bytes().to_vec())
@@ -120,7 +130,7 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
             storage.get(COL_META, KEY_BEST_BLOCK_NUMBER.as_bytes())?
         };
 
-        let crh = P::H::from(FromBytes::read(&LedgerMerkleTreeParameters::load_bytes()?[..])?);
+        let crh = P::H::from(FromBytes::read_le(&LedgerMerkleTreeParameters::load_bytes()?[..])?);
         let ledger_parameters = Arc::new(P::from(crh));
 
         match latest_block_number {
@@ -134,22 +144,24 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
 
                 let mut cm_and_indices = vec![];
 
-                for (commitment_key, index_value) in storage.get_col(COL_COMMITMENT)? {
-                    let commitment: T::Commitment = FromBytes::read(&commitment_key[..])?;
+                let cms = storage.get_col(COL_COMMITMENT)?;
+
+                for (commitment_key, index_value) in cms {
+                    let commitment: T::Commitment = FromBytes::read_le(&commitment_key[..])?;
                     let index = bytes_to_u32(&index_value) as usize;
 
                     cm_and_indices.push((commitment, index));
                 }
 
                 cm_and_indices.sort_by(|&(_, i), &(_, j)| i.cmp(&j));
-                let commitments = cm_and_indices.into_iter().map(|(cm, _)| cm);
+                let commitments = cm_and_indices.into_iter().map(|(cm, _)| cm).collect::<Vec<_>>();
 
-                let merkle_tree = MerkleTree::new(ledger_parameters.clone(), commitments)?;
+                let merkle_tree = MerkleTree::new(ledger_parameters.clone(), &commitments[..])?;
 
                 Ok(Self {
                     current_block_height: AtomicU32::new(bytes_to_u32(&val)),
                     storage,
-                    cm_merkle_tree: RwLock::new(merkle_tree),
+                    cm_merkle_tree: ArcSwap::new(Arc::new(merkle_tree)),
                     ledger_parameters,
                     _transaction: PhantomData,
                 })
@@ -157,9 +169,9 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
             None => {
                 // Add genesis block to database
 
-                let genesis_block: Block<T> = FromBytes::read(GenesisBlock::load_bytes().as_slice())?;
+                let genesis_block: Block<T> = FromBytes::read_le(GenesisBlock::load_bytes().as_slice())?;
 
-                let ledger_storage = Self::new(Some(&path.as_ref()), ledger_parameters, genesis_block)
+                let ledger_storage = Self::new(Some(path.as_ref()), ledger_parameters, genesis_block)
                     .expect("Ledger could not be instantiated");
 
                 // If there did not exist a primary ledger at the path,
@@ -174,22 +186,16 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
     }
 
     /// Attempt to catch the secondary read-only storage instance with the primary instance.
-    pub fn catch_up_secondary(&self, update_merkle_tree: bool) -> Result<(), StorageError> {
-        // Sync the secondary and primary instances
-        if self.storage.try_catch_up_with_primary().is_ok() {
-            let current_block_height_bytes = self
-                .storage
-                .get(COL_META, &KEY_BEST_BLOCK_NUMBER.as_bytes().to_vec())?
-                .ok_or_else(|| StorageError::Message("can't determine current block height".into()))?;
-            let new_current_block_height = bytes_to_u32(&current_block_height_bytes);
-            let current_block_height = self.get_current_block_height();
+    pub fn catch_up_secondary(&self, update_merkle_tree: bool, primary_height: u32) -> Result<(), StorageError> {
+        let secondary_height = self.get_current_block_height();
 
-            // If the new block height is greater than the stored block height,
-            // update the block height and merkle tree.
-            if new_current_block_height > current_block_height {
+        // If the primary block height is greater than the secondary block height, attempt to catch up,
+        // update the block height and potentially the merkle tree.
+        if primary_height > secondary_height {
+            // Sync the secondary and primary instances
+            if self.storage.try_catch_up_with_primary().is_ok() {
                 // Update the latest block height of the secondary instance.
-                self.current_block_height
-                    .store(new_current_block_height, Ordering::SeqCst);
+                self.current_block_height.store(primary_height, Ordering::SeqCst);
 
                 // Optional `cm_merkle_tree` regeneration because not all usages of
                 // the secondary instance requires it.
