@@ -14,6 +14,9 @@
 // You should have received a copy of the GNU General Public License
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
+pub mod exporter;
+pub use exporter::*;
+
 pub mod ledger;
 pub use ledger::*;
 
@@ -23,6 +26,9 @@ pub use key_value::*;
 pub mod objects;
 pub use objects::*;
 
+pub mod trim;
+pub use trim::*;
+
 pub mod mem;
 #[cfg(feature = "mem_storage")]
 pub use mem::MemDb as LedgerStorage;
@@ -31,3 +37,9 @@ pub use mem::MemDb as LedgerStorage;
 mod rocks;
 #[cfg(feature = "rocksdb_storage")]
 pub use rocks::RocksDb as LedgerStorage;
+
+pub mod validator;
+pub use validator::*;
+
+/// The number of block hashes that are returned by the `Ledger::get_block_locator_hashes` call.
+pub const NUM_LOCATOR_HASHES: u32 = 64;
