@@ -16,15 +16,15 @@
 
 use crate::*;
 use snarkvm_algorithms::traits::LoadableMerkleParameters;
-use snarkvm_objects::{errors::StorageError, DatabaseTransaction, Op, Storage, Transaction};
+use snarkvm_dpc::{errors::StorageError, DatabaseTransaction, Op, Storage, TransactionScheme};
 use snarkvm_utilities::{
     bytes::{FromBytes, ToBytes},
-    to_bytes,
+    to_bytes_le,
 };
 
-use std::collections::HashSet;
+use std::{collections::HashSet, sync::Arc};
 
-impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
+impl<T: TransactionScheme, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
     /// Get the current commitment index
     pub fn current_cm_index(&self) -> Result<usize, StorageError> {
         match self.storage.get(COL_META, KEY_CURR_CM_INDEX.as_bytes())? {
@@ -53,7 +53,7 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
     pub fn current_digest(&self) -> Result<Vec<u8>, StorageError> {
         match self.storage.get(COL_META, KEY_CURR_DIGEST.as_bytes())? {
             Some(current_digest) => Ok(current_digest),
-            None => Ok(to_bytes![self.cm_merkle_tree.read().root()].unwrap()),
+            None => Ok(to_bytes_le![self.cm_merkle_tree.load().root()].unwrap()),
         }
     }
 
@@ -110,7 +110,7 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
 
         let mut old_cm_and_indices = vec![];
         for (commitment_key, index_value) in self.storage.get_col(COL_COMMITMENT)? {
-            let commitment: T::Commitment = FromBytes::read(&commitment_key[..])?;
+            let commitment: T::Commitment = FromBytes::read_le(&commitment_key[..])?;
             let index = bytes_to_u32(&index_value) as usize;
 
             old_cm_and_indices.push((commitment, index));
@@ -119,25 +119,34 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> Ledger<T, P, S> {
         old_cm_and_indices.sort_by(|&(_, i), &(_, j)| i.cmp(&j));
         new_cm_and_indices.sort_by(|&(_, i), &(_, j)| i.cmp(&j));
 
-        let old_commitments = old_cm_and_indices.into_iter().map(|(cm, _)| cm);
-        let new_commitments = new_cm_and_indices.into_iter().map(|(cm, _)| cm);
+        let new_commitments = new_cm_and_indices.into_iter().map(|(cm, _)| cm).collect::<Vec<_>>();
 
-        let new_tree = { self.cm_merkle_tree.read().rebuild(old_commitments, new_commitments)? };
-        *self.cm_merkle_tree.write() = new_tree;
+        let merkle = self.cm_merkle_tree.load();
+        self.cm_merkle_tree.store(Arc::new(
+            merkle.rebuild(old_cm_and_indices.len(), &new_commitments[..])?,
+        ));
 
         Ok(())
     }
 
     /// Rebuild the stored merkle tree with the current stored commitments
-    pub fn update_merkle_tree(&self) -> Result<(), StorageError> {
+    pub fn update_merkle_tree(&self, new_best_block_number: u32) -> Result<(), StorageError> {
         self.rebuild_merkle_tree(vec![])?;
+        let new_digest = self.cm_merkle_tree.load().root();
 
-        let update_current_digest = DatabaseTransaction(vec![Op::Insert {
+        let mut database_transaction = DatabaseTransaction::new();
+
+        database_transaction.push(Op::Insert {
+            col: COL_DIGEST,
+            key: to_bytes_le![new_digest]?,
+            value: new_best_block_number.to_le_bytes().to_vec(),
+        });
+        database_transaction.push(Op::Insert {
             col: COL_META,
             key: KEY_CURR_DIGEST.as_bytes().to_vec(),
-            value: to_bytes![self.cm_merkle_tree.read().root()]?.to_vec(),
-        }]);
+            value: to_bytes_le![new_digest]?,
+        });
 
-        self.storage.batch(update_current_digest)
+        self.storage.batch(database_transaction)
     }
 }
