@@ -15,18 +15,17 @@
 // along with the snarkOS library. If not, see <https://www.gnu.org/licenses/>.
 
 use crate::*;
+use arc_swap::ArcSwap;
 use snarkvm_algorithms::{merkle_tree::*, traits::LoadableMerkleParameters};
-use snarkvm_dpc::LedgerError;
-use snarkvm_objects::{Block, LedgerScheme, Storage, Transaction};
+use snarkvm_dpc::{Block, LedgerError, LedgerScheme, Storage, TransactionScheme};
 use snarkvm_utilities::{
     bytes::{FromBytes, ToBytes},
-    to_bytes,
+    to_bytes_le,
 };
 
-use parking_lot::RwLock;
 use std::{fs, marker::PhantomData, path::Path, sync::Arc};
 
-impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> LedgerScheme for Ledger<T, P, S> {
+impl<T: TransactionScheme, P: LoadableMerkleParameters, S: Storage> LedgerScheme for Ledger<T, P, S> {
     type Block = Block<Self::Transaction>;
     type Commitment = T::Commitment;
     type MerkleParameters = P;
@@ -56,12 +55,12 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> LedgerScheme for L
         }
 
         let leaves: &[[u8; 32]] = &[];
-        let empty_cm_merkle_tree = MerkleTree::<Self::MerkleParameters>::new(parameters.clone(), leaves.iter())?;
+        let empty_cm_merkle_tree = MerkleTree::<Self::MerkleParameters>::new(parameters.clone(), leaves)?;
 
         let ledger_storage = Self {
             current_block_height: Default::default(),
             storage,
-            cm_merkle_tree: RwLock::new(empty_cm_merkle_tree),
+            cm_merkle_tree: ArcSwap::new(Arc::new(empty_cm_merkle_tree)),
             ledger_parameters: parameters,
             _transaction: PhantomData,
         };
@@ -83,35 +82,37 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> LedgerScheme for L
 
     /// Return a digest of the latest ledger Merkle tree.
     fn digest(&self) -> Option<Self::MerkleTreeDigest> {
-        let digest: Self::MerkleTreeDigest = FromBytes::read(&self.current_digest().unwrap()[..]).unwrap();
+        let digest: Self::MerkleTreeDigest = FromBytes::read_le(&self.current_digest().unwrap()[..]).unwrap();
         Some(digest)
     }
 
     /// Check that st_{ts} is a valid digest for some (past) ledger state.
     fn validate_digest(&self, digest: &Self::MerkleTreeDigest) -> bool {
-        self.storage.exists(COL_DIGEST, &to_bytes![digest].unwrap())
+        self.storage.exists(COL_DIGEST, &to_bytes_le![digest].unwrap())
     }
 
     /// Returns true if the given commitment exists in the ledger.
     fn contains_cm(&self, cm: &Self::Commitment) -> bool {
-        self.storage.exists(COL_COMMITMENT, &to_bytes![cm].unwrap())
+        self.storage.exists(COL_COMMITMENT, &to_bytes_le![cm].unwrap())
     }
 
     /// Returns true if the given serial number exists in the ledger.
     fn contains_sn(&self, sn: &Self::SerialNumber) -> bool {
-        self.storage.exists(COL_SERIAL_NUMBER, &to_bytes![sn].unwrap())
+        self.storage.exists(COL_SERIAL_NUMBER, &to_bytes_le![sn].unwrap())
     }
 
     /// Returns true if the given memo exists in the ledger.
-    fn contains_memo(&self, memo: &<Self::Transaction as Transaction>::Memorandum) -> bool {
-        self.storage.exists(COL_MEMO, &to_bytes![memo].unwrap())
+    fn contains_memo(&self, memo: &<Self::Transaction as TransactionScheme>::Memorandum) -> bool {
+        self.storage.exists(COL_MEMO, &to_bytes_le![memo].unwrap())
     }
 
     /// Returns the Merkle path to the latest ledger digest
     /// for a given commitment, if it exists in the ledger.
     fn prove_cm(&self, cm: &Self::Commitment) -> anyhow::Result<Self::MerklePath> {
-        let cm_index = self.get_cm_index(&to_bytes![cm]?)?.ok_or(LedgerError::InvalidCmIndex)?;
-        let result = self.cm_merkle_tree.read().generate_proof(cm_index, cm)?;
+        let cm_index = self
+            .get_cm_index(&to_bytes_le![cm]?)?
+            .ok_or(LedgerError::InvalidCmIndex)?;
+        let result = self.cm_merkle_tree.load().generate_proof(cm_index, cm)?;
 
         Ok(result)
     }
@@ -124,6 +125,6 @@ impl<T: Transaction, P: LoadableMerkleParameters, S: Storage> LedgerScheme for L
         cm: &Self::Commitment,
         witness: &Self::MerklePath,
     ) -> bool {
-        witness.verify(&digest, cm).unwrap()
+        witness.verify(digest, cm).unwrap()
     }
 }
